/* 

   $Id: getaddrinfo.c,v 1.1.1.1 1996/10/31 12:51:32 pan Exp $

      Implementation of getaddrinfo using /etc/hosts file only

   This version only takes care of the name and service parameters and
   returns a struct sockaddr

 */

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>

#include <stdio.h>
#include <netdb.h>
#include <resolv.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>

extern struct hostent * getnexthost(int close);

int getaddrinfo(char *name, char *service,
		struct addrinfo *req,
		struct addrinfo **pai)
{
	int family;
	struct addrinfo *result;
	struct sockaddr_in6 sock, *s;
	struct sockaddr_in *s4 = (struct sockaddr_in *) &sock;
	register struct servent *p;
	register struct hostent *q;
	char *p_addr;
	int i;

	if ((name == NULL) && (service == NULL)) {	/*error */
		return (EAI_NONAME);
	}
	if ((result =
	  (struct addrinfo *) malloc(sizeof(struct addrinfo))) == NULL) {
		return (EAI_MEMORY);
	}
	result->ai_next = NULL;
	result->ai_canonname = NULL;
	result->ai_flags = 0;
	result->ai_socktype = 0;
	
	/* req->ai_flag must be proceeded  TO DO */
	
	if (req == NULL) {
		family = PF_UNSPEC;
		result->ai_protocol = 0;
	} else {
		result->ai_family = req->ai_family;
		result->ai_protocol = req->ai_protocol;
	}
	
	family = result->ai_family;
	if ((family != AF_INET) && (family != AF_INET6) &&
	    (family != PF_UNSPEC)) {
		free(result);
		return (EAI_FAMILY);
	}
	
	if (service != NULL) {
		if (p = getserbyname(service, result->ai_protocol))
				sock.sin6_port = p->s_port;
		else {
			  free(result);
			  return (EAI_SERVICE);
		}
	}	/* if service != NULL */
	
	memcpy(s, &sock, sizeof(sock));
	result->ai_addr = (struct sockaddr *) s;
	*pai = result;
	return (0);
}				/* getaddrinfo */



void freeaddrinfo(struct addrinfo *ai)
{
	if (ai->ai_next != NULL) {
		freeaddrinfo(ai->ai_next);
	} else {
		if (ai->ai_addr != NULL) {
			free(ai->ai_addr);
		}
		free(ai);
	}
}				/* freeaddrinfo */
